#include "util.h"

#ifndef lint
char copyright[] =
"CONVERT_APPL - Version "VERSION"\nCopyright (c) 1995 Brian A. Lantz. All rights reserved.\n";
#endif /* not lint */


char *getargp;

char *getarg (char *line);
time_t parsetime (char *tmstr,int dob);


char *getarg(char *line)
{
char *arg;

	if (line)
		getargp = line;
	arg = getargp;
	while (*getargp && *getargp != '|') {
		getargp++;
	}
	if (*getargp)
		*getargp++ = '\0';
	return arg;
}

char const *months[12] = {
	"Jan", "Feb", "Mar", "Apr", "May", "Jun",
	"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
};


time_t
parsetime(char *tmstr,int dob)
{
struct tm t;
char month[8], ampm[8];
int k, nerf;

	/* daily updates give date/time as 'yymmdd' */
	if (tmstr && strlen(tmstr) == 6)	{
		t.tm_hour = t.tm_min = t.tm_sec = 0;
		strncpy (month, tmstr, 2);
		t.tm_year = atoi(month);
		if (!dob && t.tm_year < 50)
			t.tm_year += 100;
		strncpy (month, &tmstr[2], 2);
		t.tm_mon = atoi(month) - 1;
		strncpy (month, &tmstr[4], 2);
		t.tm_mday = atoi(month);
		return (mktime(&t));
	}

	/* full database gives date/time as 'Mar  9 1993 12:00:00:000AM' */
	sscanf (tmstr, "%s %d %d %2d:%2d:%2d:%3d%s", month, &t.tm_mday, &t.tm_year,
		&t.tm_hour, &t.tm_min, &t.tm_sec, &nerf, ampm);
	t.tm_year -= 1900;
	if (toupper(ampm[0]) == 'P')	{
		if (t.tm_hour < 12)
			t.tm_hour += 12;
	} else	{
		if (t.tm_hour == 12)
			t.tm_hour = 0;
	}
	t.tm_mon = 0;
	for (k = 0; k < 12; k++)	{
		if (!strcasecmp(months[k], month))	{
			t.tm_mon = k;
			break;
		}
	}
#if 0
	printf ("%s = %d/%d/%d %d:%d:%d\n", tmstr, t.tm_mon + 1, t.tm_mday, t.tm_year + 1900,
		t.tm_hour, t.tm_min, t.tm_sec);
#endif
	return (mktime (&t));
}


int
main (void)
{
long total = 0, skipped = 0;
FILE *fp;
GDBM_FILE db;
datum key_data, data_data, return_data;
struct appl_dat ap;
char buffer[1024];
char callsign[2048];
char type_purpose[8], type_applicant;

	if ((fp = fopen(_PATH_APPL_FILE, "r")) == NULLFILE)	{
		printf ("Error #%d opening file %s\n", errno, _PATH_APPL_FILE);
		return 0;
	}
	db = gdbm_open (_PATH_DATABASE, 0, GDBM_WRCREAT | GDBM_FAST, 00664, NULL);
	if (db == NULL)	{
		printf ("Error setting up gdbm file: %s\n", gdbm_strerror(gdbm_errno));
		return 0;
	}
	while (!feof(fp))	{
		fgets (buffer, 1024, fp);
		if (feof(fp))
			continue;
		getargp = buffer;
		strcpy (callsign, getarg(0));
		key_data.dptr = callsign;
		key_data.dsize = strlen(callsign) + 1;

		return_data = gdbm_fetch (db, key_data);
		if (return_data.dptr != NULL)	{
			(void) unsqueeze (&ap, return_data.dptr);
			free (return_data.dptr);
		} else	{
			ap.type_op_class = TYPE_OP_UNKNOWN;
			memset (ap.callsign_prev, ' ', 10);
			ap.callsign_prev[0] = 0;
			ap.type_prev_op_class = TYPE_OP_UNKNOWN;
		}
		/* discard id_file_num */
		(void) getarg(0);
		/* look at but not store the type_purpose */
		strcpy (type_purpose, getarg(0));
		/* look at but not store the type_applicant */
		type_applicant = *(getarg(0));
		if (type_applicant != TYPE_APP_INDIVIDUAL)	{
			printf ("Skipping an non-individual entry: %s\n", callsign);
			skipped++;
			continue;
		}
		memset (ap.name_licensee, ' ', 40);
		strcpy (ap.name_licensee, getarg(0));
		memset (ap.text_street, ' ', 35);
		strcpy (ap.text_street, getarg(0));
		memset (ap.text_pobox, ' ', 20);
		strcpy (ap.text_pobox, getarg(0));
		memset (ap.city, ' ', 20);
		strcpy (ap.city, getarg(0));
		memset (ap.state, ' ', 2);
		strcpy (ap.state, getarg(0));
		memset (ap.zipcode, ' ', 9);
		strcpy (ap.zipcode, getarg(0));
		ap.date_of_birth = parsetime(getarg(0), 1);
		ap.date_issue = parsetime(getarg(0), 0);
		ap.date_expire = parsetime(getarg(0), 0);
		ap.date_last_change = parsetime(getarg(0), 0);
		/* discard the id_examiner, flag_renewal_notice, and date_renewal_notice */

		data_data.dsize = squeeze (&ap, buffer);
		data_data.dptr = buffer;
		total++;
		if (gdbm_store (db, key_data, data_data, GDBM_REPLACE) != 0)	{
			printf ("Item %s not inserted\n", callsign);
			skipped++;
		} else
			printf ("Adding information for %s\n", callsign);
	}
	printf ("Complete! There were %ld entries added - %ld skipped\n", total - skipped, skipped);
	gdbm_close (db);
	return (0);
}

